from enum import Enum


# 图像类型枚举类
class GraphTypeEnum(Enum):
    GRAPHTYPE_NULL = 0  # 无效
    GRAPHTYPE_DEPTH = 1  # 深度
    GRAPHTYPE_POINTCLOUD = 2  # 点云
    GRAPHTYPE_IR = 3  # IR
    GRAPHTYPE_RGB = 4  # RGB
    GRAPHTYPE_RGBDDEPTH = 5  # RGBD深度
    GRAPHTYPE_RGBDPOINTCLOUD = 6  # RGBD点云
    GRAPHTYPE_RGBDRGB = 7  # RGBD的RGB


# 流类型枚举类
class StreamTypeEnum(Enum):
    STREAMTYPE_NULL = 0  # 无效
    STREAMTYPE_RAW = 1  # RAW
    STREAMTYPE_DEPTH = 2  # 深度
    STREAMTYPE_RGB = 3  # RGB
    STREAMTYPE_DEPTHIR = 4  # 深度+IR
    STREAMTYPE_DEPTHRGB = 5  # 深度+RGB
    STREAMTYPE_DEPTHIRRGB = 6  # 深度+IR+RGB
    STREAMTYPE_RGBD = 7  # RGBD(mapping后的深度+RGB)


# 数据类型枚举类
class DataTypeEnum(Enum):
    DATATYPE_NULL = 0  # 无效
    DATATYPE_DEPTH = 1  # 深度
    DATATYPE_DEPTHCOLOR = 2  # 深度彩色图像
    DATATYPE_POINTCLOUD = 3  # 点云
    DATATYPE_IR = 4  # IR
    DATATYPE_RGB = 5  # RGB
    DATATYPE_RGBDDEPTH = 6  # RGBD深度
    DATATYPE_RGBDDEPTHCOLOR = 7  # RGBD深度彩色图像
    DATATYPE_RGBDPOINTCLOUD = 8  # RGBD点云
    DATATYPE_RGBDRGB = 9  # RGBD的RGB


# 设备类型枚举类
class DeviceTypeEnum(int):
    DEVICETYPE_NULL = 0  # 无效
    DEVICETYPE_CS30_DUAL = 1  # CS30双频
    DEVICETYPE_CS30_SINGLE = 2  # CS30单频
    DEVICETYPE_CS20_DUAL = 3  # CS20双频
    DEVICETYPE_CS20_SINGLE = 4  # CS20单频
    DEVICETYPE_CS20_P = 5,  # CS20P
    DEVICETYPE_CS40 = 6,  # cs40
    DEVICETYPE_TOTALCOUNT = 7  # 设备类型总数


# 分辨率信息
class ResolutionInfo(object):
    def __init__(self):
        self.m_nWidth = 0
        self.m_nHeight = 0
        self.m_nResloutionCode = 0


class DeviceInfo(object):
    def __init__(self):
        self.m_nDeviceId = 0
        self.m_nDeviceType = DeviceTypeEnum.DEVICETYPE_NULL


class DeviceParam(object):
    def __init__(self):
        self.m_nDeviceID = 1  # device Id(设备ID)
        self.m_nDeviceType = 0  # device type(设备类型)
        self.m_strDeviceName = ''  # device name(设备名称)
        self.m_strDeviceSN = ''  # device serial number(设备SN)
        self.m_strDeviceHWVersion = ''  # Device firmware version(设备固件版本)
        self.m_mapLimitResolutionInfo = dict()  # Device Available Resolution(设备可用分辨率)
        self.m_mapCurrentResolutionInfo = dict()  # The resolution currently used by the device(设备当前使用分辨率)
        self.m_bFilter = True  # Filter on flag(滤波开启标志)
        self.m_vecFilterList = []  # Filter List(滤波列表)
        self.m_bMirror = False  # Horizontal Mirror on flag(水平镜像开启标志)
        self.m_bUndistort = False  # Undistort removal on flag(去畸变开启标志)
        self.m_bFlip = False  # Vertical flip open flag(垂直翻转开启标志)
        self.m_nIntegralTimeRangeMin = 0  # Minimum value of integration time range(积分时间范围最小值)
        self.m_nIntegralTimeRangeMax = 3000  # Maximum value of integration time range(积分时间范围最大值)
        self.m_nIntegralTime = 0  # Current integration time(当前积分时间)
        self.m_nDistanceRangeMin = 0  # Range Minimum(量程范围最小值)
        self.m_nDistanceRangeMax = 3000  # Range Maximum(量程范围最大值)
        self.m_nDistanceMin = 0  # Current range minimum(当前量程最小值)
        self.m_nDistanceMax = 3000  # Current range maximum(当前量程最大值)
        self.m_openedStream = StreamTypeEnum.STREAMTYPE_NULL  # Current open stream type(当前已开流类型)
