import os
from ctypes import *

import DataDefine
import ErrorCodeEnum as ErrCode
import SYDataDefine
from DataDefine import DeviceParam

lib_path = './dll/SynexensSDK.dll'
dllLib = cdll.LoadLibrary(lib_path)
if dllLib == 0:
    print(" Could not open file DLL")

os.chdir("./dll/")
class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


class LogTypeEnum(int):
    DEBUG_T = 0,
    WARNING_T = 1,
    ERROR_T = 2


def print_t(log_type, text):
    if log_type is LogTypeEnum.DEBUG_T:
        print(f"\r\n{bcolors.OKGREEN} {text} {bcolors.ENDC}\r\n")
    elif log_type is LogTypeEnum.WARNING_T:
        print(f"\r\n{bcolors.WARNING} {text} {bcolors.ENDC}\r\n")
    elif log_type is LogTypeEnum.ERROR_T:
        print(f"\r\n{bcolors.FAIL} {text} {bcolors.ENDC}\r\n")


class DeviceControl(object):
    def __init__(self):
        self.init_sdk()
        self.m_vecDeviceInfo: [DataDefine.DeviceInfo] = list()  # 查找到的设备信息集合
        self.m_mapDeviceInfo: {int, DataDefine.DeviceInfo} = dict()  # 查找到的设备信息集合
        self.m_mapDeviceParam: {int, DataDefine.DeviceParam} = dict()  # 已连接的设备参数集合

    # 初始化SDK
    def init_sdk(self) -> ErrCode.StatusCodeEnum:
        errorCode = dllLib.InitSDK()
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 反初始化SDK
    def uninit_sdk(self) -> ErrCode.StatusCodeEnum:
        errorCode = dllLib.UnInitSDK()
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    def convert_device_type_local_to_sdk(self, deviceType: DataDefine.DeviceTypeEnum) -> SYDataDefine.SYDeviceTypeEnum:
        sdkDeviceType = SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_NULL
        if deviceType == DataDefine.DeviceTypeEnum.DEVICETYPE_CS30_DUAL:
            sdkDeviceType = SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS30_DUAL
        elif deviceType == DataDefine.DeviceTypeEnum.DEVICETYPE_CS30_SINGLE:
            sdkDeviceType = SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS30_SINGLE
        elif deviceType == DataDefine.DeviceTypeEnum.DEVICETYPE_CS20_DUAL:
            sdkDeviceType = SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS20_DUAL
        elif deviceType == DataDefine.DeviceTypeEnum.DEVICETYPE_CS20_SINGLE:
            sdkDeviceType = SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS20_SINGLE
        elif deviceType == DataDefine.DeviceTypeEnum.DEVICETYPE_CS20_P:
            sdkDeviceType = SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS20_P
        elif deviceType == DataDefine.DeviceTypeEnum.DEVICETYPE_CS40:
            sdkDeviceType = SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS40
        return sdkDeviceType

    def convert_device_type_sdk_to_local(self, deviceType: SYDataDefine.SYDeviceTypeEnum) -> DataDefine.DeviceTypeEnum:
        guiDeviceType = DataDefine.DeviceTypeEnum.DEVICETYPE_NULL
        if deviceType.value == SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS30_DUAL:
            guiDeviceType = DataDefine.DeviceTypeEnum.DEVICETYPE_CS30_DUAL
        elif deviceType.value == SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS30_SINGLE:
            guiDeviceType = DataDefine.DeviceTypeEnum.DEVICETYPE_CS30_SINGLE
        elif deviceType.value == SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS20_DUAL:
            guiDeviceType = DataDefine.DeviceTypeEnum.DEVICETYPE_CS20_DUAL
        elif deviceType.value == SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS20_SINGLE:
            guiDeviceType = DataDefine.DeviceTypeEnum.DEVICETYPE_CS20_SINGLE
        elif deviceType.value == SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS20_P:
            guiDeviceType = DataDefine.DeviceTypeEnum.DEVICETYPE_CS20_P
        elif deviceType.value == SYDataDefine.SYDeviceTypeEnum.SYDEVICETYPE_CS40:
            guiDeviceType = DataDefine.DeviceTypeEnum.DEVICETYPE_CS40
        return guiDeviceType

    # 查找设备
    def find_device(self, list_device: list[DataDefine.DeviceInfo]) -> ErrCode.StatusCodeEnum:
        self.m_vecDeviceInfo.clear()
        self.m_mapDeviceInfo.clear()

        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        # 设置FindDevice的形参类型
        dllLib.FindDevice.argtypes = [POINTER(c_int32), POINTER(SYDataDefine.SYDeviceInfo)]

        nDeviceCount = c_int32()  # 设备数量
        errorCode = dllLib.FindDevice(byref(nDeviceCount), None)
        print_t(LogTypeEnum.DEBUG_T, "查找到的设备数量:{}".format(nDeviceCount.value))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            if nDeviceCount.value <= 0:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_DEVICELISTEMPTY
            else:
                pDeviceInfo = (SYDataDefine.SYDeviceInfo * nDeviceCount.value)()
                errorCode = dllLib.FindDevice(nDeviceCount, pDeviceInfo)
                if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
                    for i in range(nDeviceCount.value):
                        deviceInfo = DataDefine.DeviceInfo()
                        print_t(LogTypeEnum.DEBUG_T, "设备ID:{} 设备类型:{}".format(pDeviceInfo[i].m_nDeviceID, pDeviceInfo[i].m_deviceType.value))
                        deviceInfo.m_nDeviceId = pDeviceInfo[i].m_nDeviceID
                        deviceInfo.m_nDeviceType = self.convert_device_type_sdk_to_local(pDeviceInfo[i].m_deviceType)
                        list_device.append(deviceInfo)
                        self.m_mapDeviceInfo[deviceInfo.m_nDeviceId] = deviceInfo
                    self.m_vecDeviceInfo = list_device
                    errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
                else:
                    errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED

        return errorCodeRet

    # 打开设备
    def open_device(self, nDeviceId: int) -> ErrCode.StatusCodeEnum:
        dllLib.OpenDevice.argtypes = [POINTER(SYDataDefine.SYDeviceInfo)]
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        itFind = self.m_mapDeviceInfo.get(nDeviceId, "NULL")
        if itFind == "NULL":
            return errorCodeRet

        deviceInfo = SYDataDefine.SYDeviceInfo()
        deviceInfo.m_nDeviceID = itFind.m_nDeviceId
        deviceInfo.m_deviceType = self.convert_device_type_local_to_sdk(itFind.m_nDeviceType)

        errorCodeOpenDevice = dllLib.OpenDevice(deviceInfo)
        if errorCodeOpenDevice == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            _deviceParam = DataDefine.DeviceParam()
            _deviceParam.m_nDeviceID = itFind.m_nDeviceId
            _deviceParam.m_nDeviceType = itFind.m_nDeviceType
            if self.get_divce_sn(_deviceParam) is not ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_GETSNFAILED
                return errorCodeRet
            if self.get_device_hw_version(_deviceParam) is not ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_GETHWVERSIONFAILED
                return errorCodeRet
            if self.get_device_support_resolution(_deviceParam) is not ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_GETSUPPORTRESOLUTIONFAILED
                return errorCodeRet
            if self.get_device_filter(_deviceParam) is not ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_GETFILTERFAILED
                return errorCodeRet
            if self.get_device_mirror(_deviceParam) is not ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_GETMIRRORFAILED
                return errorCodeRet
            if self.get_device_flip(_deviceParam) is not ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_GETFLIPFAILED
                return errorCodeRet
            if self.get_device_distance_range(_deviceParam) is not ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_GETDISTANCERANGEFAILED
                return errorCodeRet
            if self.get_device_distance(_deviceParam) is not ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_GETDISTANCEFAILED
                return errorCodeRet
            if self.get_device_filter_list(_deviceParam) is not ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_GETFILTERLISTFAILED
                return errorCodeRet
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
            self.m_mapDeviceParam[_deviceParam.m_nDeviceID] = _deviceParam
        return errorCodeRet

    # 关闭设备
    def close_device(self, nDeviceId: int) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        deviceInfo = self.m_mapDeviceInfo.get(nDeviceId, "NULL")
        if deviceInfo == "NULL":
            return ErrCode.StatusCodeEnum.ERRORCODE_DEVICENOTEXIST
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        errorCode = dllLib.CloseDevice(c_uint(nDeviceId))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet

    # 获取设备参数
    def get_device_param(self, nDeviceId: int, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        devParam = self.m_mapDeviceParam.get(nDeviceId, "NULL")
        if devParam == "NULL":
            return ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS

    # 获取当前流类型
    def get_current_stream_type(self, nDeviceId: int) -> DataDefine.StreamTypeEnum:
        sdkStreamType = dllLib.GetCurrentStreamType(c_uint(nDeviceId))
        if sdkStreamType == SYDataDefine.SYStreamTypeEnum.SYSTREAMTYPE_RAW:
            return DataDefine.StreamTypeEnum.STREAMTYPE_RAW
        elif sdkStreamType == SYDataDefine.SYStreamTypeEnum.SYSTREAMTYPE_DEPTH:
            return DataDefine.StreamTypeEnum.STREAMTYPE_DEPTH
        elif sdkStreamType == SYDataDefine.SYStreamTypeEnum.SYSTREAMTYPE_RGB:
            return DataDefine.StreamTypeEnum.STREAMTYPE_RGB
        elif sdkStreamType == SYDataDefine.SYStreamTypeEnum.SYSTREAMTYPE_DEPTHIR:
            return DataDefine.StreamTypeEnum.STREAMTYPE_DEPTHIR
        elif sdkStreamType == SYDataDefine.SYStreamTypeEnum.SYSTREAMTYPE_DEPTHRGB:
            return DataDefine.StreamTypeEnum.STREAMTYPE_RGB
        elif sdkStreamType == SYDataDefine.SYStreamTypeEnum.SYSTREAMTYPE_DEPTHIRRGB:
            return DataDefine.StreamTypeEnum.STREAMTYPE_DEPTHIRRGB
        elif sdkStreamType == SYDataDefine.SYStreamTypeEnum.SYSTREAMTYPE_RGBD:
            return DataDefine.StreamTypeEnum.STREAMTYPE_RGBD
        else:
            return DataDefine.StreamTypeEnum.STREAMTYPE_NULL

    # 设置分辨率
    def set_resolution(self, nDeviceId: int, graphType: DataDefine.GraphTypeEnum, resolutionInfo: DataDefine.ResolutionInfo) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        frameType = SYDataDefine.SYFrameTypeEnum.SYFRAMETYPE_NULL
        if graphType == DataDefine.GraphTypeEnum.GRAPHTYPE_RGBDDEPTH:
            itFindDeviceParam = self.m_mapDeviceParam.get(nDeviceId, "NULL")
            if itFindDeviceParam == "NULL":
                return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
            else:
                itFindResolutionInfo = itFindDeviceParam.m_mapCurrentResolutionInfo.get(graphType, "NULL")
                if itFindResolutionInfo == "NULL":
                    itFindDeviceParam.m_mapCurrentResolutionInfo[graphType] = resolutionInfo
                else:
                    itFindResolutionInfo = resolutionInfo
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        elif graphType == DataDefine.GraphTypeEnum.GRAPHTYPE_DEPTH:
            frameType = SYDataDefine.SYFrameTypeEnum.SYFRAMETYPE_DEPTH
        elif graphType == DataDefine.GraphTypeEnum.GRAPHTYPE_IR:
            frameType = SYDataDefine.SYFrameTypeEnum.SYFRAMETYPE_IR
        elif graphType == DataDefine.GraphTypeEnum.GRAPHTYPE_RGB:
            frameType = SYDataDefine.SYFrameTypeEnum.SYFRAMETYPE_RGB
        else:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNGRAPHTYPE

        errorCode = dllLib.SetFrameResolution(c_uint(nDeviceId), c_int(frameType.value), c_int(resolutionInfo.m_nResloutionCode))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            itFindDeviceParam = self.m_mapDeviceParam.get(nDeviceId, "NULL")
            if itFindDeviceParam == "NULL":
                return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
            else:
                itFindResolutionInfo = itFindDeviceParam.m_mapCurrentResolutionInfo.get(graphType, "NULL")
                if itFindResolutionInfo == "NULL":
                    itFindDeviceParam.m_mapCurrentResolutionInfo[graphType] = resolutionInfo
                else:
                    itFindResolutionInfo = resolutionInfo
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 设置滤波
    def set_filter(self, nDeviceId: int, bFilter: bool) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCode = dllLib.SetFilter(c_uint(nDeviceId), c_bool(bFilter))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 设置畸变
    def set_undistort(self, nDeviceId: int, bUndistort: bool) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCode = dllLib.SetUndistort(c_uint(nDeviceId), c_bool(bUndistort))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 设置滤波列表
    def set_filter_list(self, nDeviceId: int, nIndex: int, nFilterType: SYDataDefine.SYFilterTypeEnum,
                        bAdd: bool) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCode = SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS
        if bAdd:
            errorCode = dllLib.AddFilter(c_uint(nDeviceId), c_int(nFilterType))
            devFind = self.m_mapDeviceParam.get(nDeviceId, "NULL")
            devFind.m_vecFilterList.append(nFilterType)
        else:
            errorCode = dllLib.DeleteFilter(c_uint(nDeviceId), c_int(nIndex))
            devFind = self.m_mapDeviceParam.get(nDeviceId, "NULL")
            devFind.m_vecFilterList.pop(nIndex)
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 设置滤波参数
    def set_filter_param(self, nDeviceId: int, nFilterType: SYDataDefine.SYFilterTypeEnum, nParamCount: int,
                         pFilterParamList: list[int]):
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID

        pFltParam = (c_float * nParamCount)()

        for i in range(nParamCount):
            pFltParam[i] = c_float(pFilterParamList[i])

        errorCode = dllLib.SetFilterParam(c_uint(nDeviceId), c_int(nFilterType), c_int(nParamCount), pFltParam)
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 设置水平镜像
    def set_mirror(self, nDeviceId: int, bMirror: bool) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCode = dllLib.SetMirror(c_uint(nDeviceId), c_bool(bMirror))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 设置垂直翻转
    def set_flip(self, nDeviceId: int, bFlip: bool) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCode = dllLib.SetFlip(c_uint(nDeviceId), c_bool(bFlip))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 设置积分时间
    def set_integral_time(self, nDeviceId: int, nIntergralTime: int) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCode = dllLib.SetIntegralTime(c_uint(nDeviceId), c_int(nIntergralTime))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 设置测量范围
    def set_distance(self, nDeviceId: int, nDistanceMin: int, nDistanceMax: int):
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCode = dllLib.SetDistanceUserRange(c_uint(nDeviceId), c_int(nDistanceMin), c_int(nDistanceMax))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 获取设备SN号
    def get_divce_sn(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        nStrLength = c_int(0)
        errorCodeGetDeviceSN = dllLib.GetDeviceSN(c_uint(devParam.m_nDeviceID), byref(nStrLength), None)
        if errorCodeGetDeviceSN == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            if nStrLength.value > 0:
                pStringDeviceSN = (c_char * nStrLength.value)()
                errorCodeGetDeviceSN = dllLib.GetDeviceSN(c_uint(devParam.m_nDeviceID), byref(nStrLength),
                                                          pStringDeviceSN)
                if errorCodeGetDeviceSN == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
                    devParam.m_strDeviceSN = pStringDeviceSN.value
                    errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
                    print("SN:" + str(devParam.m_strDeviceSN))
                else:
                    errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
                del pStringDeviceSN
            else:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED

        return errorCodeRet

    # 获取设备固件版本号
    def get_device_hw_version(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        nStrLength = c_int(0)
        errorCodeGetHWVersion = dllLib.GetDeviceHWVersion(c_uint(devParam.m_nDeviceID), byref(nStrLength), None)
        if errorCodeGetHWVersion == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            if nStrLength.value > 0:
                pStringFWVersion = (c_char * nStrLength.value)()
                errorCodeGetHWVersion = dllLib.GetDeviceHWVersion(c_uint(devParam.m_nDeviceID), byref(nStrLength),
                                                                  pStringFWVersion)
                if errorCodeGetHWVersion == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
                    print("固件版本号:" + str(pStringFWVersion.value))
                    devParam.m_strDeviceHWVersion = str(pStringFWVersion.value)
                    errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
                else:
                    errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
            else:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED

        return errorCodeRet

    # 获取设备积分时间设置范围
    def get_device_integral_time_range(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        if not devParam.m_mapCurrentResolutionInfo:
            return ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        resloutionInfo = devParam.m_mapCurrentResolutionInfo.get(DataDefine.GraphTypeEnum.GRAPHTYPE_DEPTH)
        errorCodeIntegralTime = dllLib.GetIntegralTimeRange(c_uint(devParam.m_nDeviceID),
                                                            c_int(resloutionInfo.m_nResolutionCode),
                                                            c_int(devParam.m_nIntegralTimeRangeMin),
                                                            c_int(devParam.m_nIntegralTimeRangeMax))
        print("m_nIntegralTimeRangeMin:" + str(devParam.m_nIntegralTimeRangeMin) + " m_nIntegralTimeRangeMax:" + str(
            devParam.m_nIntegralTimeRangeMax))
        if errorCodeIntegralTime == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 获取设备当前积分时间
    def get_device_integral_time(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeIntegralTime = dllLib.GetIntegralTime(c_uint(devParam.m_nDeviceID),
                                                       byref(c_int(devParam.m_nIntegralTime)))
        if errorCodeIntegralTime == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 获取设备支持的分辨率
    def get_device_support_resolution(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        nSupportTypeCount = c_int(0)
        errorCodeQueryFrameType = dllLib.QueryDeviceSupportFrameType(c_uint(devParam.m_nDeviceID),
                                                                     byref(nSupportTypeCount), None)
        if errorCodeQueryFrameType == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code and nSupportTypeCount.value > 0:
            pSupportType = (SYDataDefine.SYSupportTypeEnum * nSupportTypeCount.value)()
            errorCodeQueryFrameType = dllLib.QueryDeviceSupportFrameType(devParam.m_nDeviceID, byref(nSupportTypeCount),
                                                                         pSupportType)
            if errorCodeQueryFrameType == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code and nSupportTypeCount.value > 0:
                setLimitGraph: set[DataDefine.GraphTypeEnum] = set()
                setLimitGraph.clear()
                for i in range(nSupportTypeCount.value):
                    if pSupportType[i].value == SYDataDefine.SYSupportTypeEnum.SYSUPPORTTYPE_DEPTH:
                        setLimitGraph.add(DataDefine.GraphTypeEnum.GRAPHTYPE_DEPTH)
                    elif pSupportType[i].value == SYDataDefine.SYSupportTypeEnum.SYSUPPORTTYPE_RGB:
                        setLimitGraph.add(DataDefine.GraphTypeEnum.GRAPHTYPE_RGB)
                    elif pSupportType[i].value == SYDataDefine.SYSupportTypeEnum.SYSUPPORTTYPE_RGBD:
                        setLimitGraph.add(DataDefine.GraphTypeEnum.GRAPHTYPE_RGBDDEPTH)
                devParam.m_mapLimitResolutionInfo.clear()
                for graphType in setLimitGraph:
                    vecLimitResolution: list[DataDefine.ResolutionInfo] = list()
                    if self.get_device_usable_resolution(devParam.m_nDeviceID, graphType,
                                                         vecLimitResolution) == ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS:
                        devParam.m_mapLimitResolutionInfo[graphType] = vecLimitResolution
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
            else:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
            del pSupportType
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet

    # 获取设备滤波开关状态
    def get_device_filter(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        bFilter = c_bool()
        errorCode = dllLib.GetFilter(c_uint(devParam.m_nDeviceID), byref(bFilter))
        devParam.m_bFilter = bFilter.value
        print("m_bFilter:" + str(devParam.m_bFilter))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 获取设备滤波列表
    def get_device_filter_list(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED

        nFilterListCount = c_int(0)
        errorCode = dllLib.GetFilterList(c_uint(devParam.m_nDeviceID), byref(nFilterListCount), None)

        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            if nFilterListCount.value > 0:
                pFilterTypeList = (SYDataDefine.SYFilterTypeEnum * nFilterListCount.value)()
                errorCode = dllLib.GetFilterList(c_uint(devParam.m_nDeviceID), byref(nFilterListCount), pFilterTypeList)
                if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
                    for j in range(nFilterListCount.value):
                        devParam.m_vecFilterList.insert(SYDataDefine.SYFilterTypeEnum(pFilterTypeList[j]))
                    errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
                else:
                    errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED

        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS

        return errorCodeRet

    # 获取设备水平镜像状态
    def get_device_mirror(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        bMirror = c_bool()
        errorCode = dllLib.GetMirror(c_uint(devParam.m_nDeviceID), byref(bMirror))
        devParam.m_bMirror = bMirror.value
        print("m_bMirror:" + str(devParam.m_bMirror))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 获取设备垂直翻转状态
    def get_device_flip(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        bFlip = c_bool()
        errorCode = dllLib.GetFlip(c_uint(devParam.m_nDeviceID), byref(bFlip))
        devParam.m_bFlip = bFlip.value
        print("m_bFlip:" + str(devParam.m_bFlip))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 获取设备测距量程
    def get_device_distance_range(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        nDistanceRangeMin, nDistanceRangeMax = c_int(), c_int()
        errorCode = dllLib.GetDistanceMeasureRange(c_uint(devParam.m_nDeviceID), byref(nDistanceRangeMin),
                                                   byref(nDistanceRangeMax))
        devParam.m_nDistanceRangeMin, devParam.m_nDistanceRangeMax = nDistanceRangeMin.value, nDistanceRangeMax.value
        print("m_nDistanceRangeMin:" + str(devParam.m_nDistanceRangeMin) + " m_nDistanceRangeMax:" + str(
            devParam.m_nDistanceRangeMax))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 获取设备测距范围
    def get_device_distance(self, devParam: DeviceParam) -> ErrCode.StatusCodeEnum:
        if devParam.m_nDeviceID == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        nDistanceMin, nDistanceMax = c_int(), c_int()
        errorCode = dllLib.GetDistanceUserRange(c_uint(devParam.m_nDeviceID), byref(nDistanceMin), byref(nDistanceMax))
        devParam.m_nDistanceMin, devParam.m_nDistanceMax = nDistanceMin.value, nDistanceMax.value
        print("m_nDistanceMin:" + str(devParam.m_nDistanceMin) + " m_nDistanceMax:" + str(devParam.m_nDistanceMax))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 获取设备可用分辨率
    def get_device_usable_resolution(self, nDeviceId: int, graphType: DataDefine.GraphTypeEnum, listLimitResolution: list[DataDefine.ResolutionInfo]) -> tuple[ErrCode.StatusCodeEnum, list]:
        sdkSupportType = SYDataDefine.SYSupportTypeEnum.SYSUPPORTTYPE_NULL
        if graphType == DataDefine.GraphTypeEnum.GRAPHTYPE_DEPTH:
            sdkSupportType = SYDataDefine.SYSupportTypeEnum.SYSUPPORTTYPE_DEPTH
        elif graphType == DataDefine.GraphTypeEnum.GRAPHTYPE_RGB:
            sdkSupportType = SYDataDefine.SYSupportTypeEnum.SYSUPPORTTYPE_RGB
        elif graphType == DataDefine.GraphTypeEnum.GRAPHTYPE_RGBDDEPTH:
            sdkSupportType = SYDataDefine.SYSupportTypeEnum.SYSUPPORTTYPE_RGBD
        else:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNGRAPHTYPE, list()

        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        nResolutionCount = c_int(0)
        errorCodeQueryResolution = dllLib.QueryDeviceSupportResolution(c_uint(nDeviceId), sdkSupportType,
                                                                       byref(nResolutionCount), None)
        if errorCodeQueryResolution == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code and nResolutionCount.value > 0:
            pResolution = (SYDataDefine.SYResolutionEnum * nResolutionCount.value)()
            errorCodeQueryResolution = dllLib.QueryDeviceSupportResolution(c_uint(nDeviceId), sdkSupportType,
                                                                           byref(nResolutionCount), pResolution)
            if errorCodeQueryResolution == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code and nResolutionCount.value > 0:
                listLimitResolution.clear()
                for k in range(nResolutionCount.value):
                    resolutionInfo = DataDefine.ResolutionInfo()
                    resolutionInfo.m_nResloutionCode = pResolution[k].value
                    if pResolution[k].value == SYDataDefine.SYResolutionEnum.SYRESOLUTION_320_240:
                        resolutionInfo.m_nWidth = 320
                        resolutionInfo.m_nHeight = 240
                    elif pResolution[k].value == SYDataDefine.SYResolutionEnum.SYRESOLUTION_640_480:
                        resolutionInfo.m_nWidth = 640
                        resolutionInfo.m_nHeight = 480
                    elif pResolution[k].value == SYDataDefine.SYResolutionEnum.SYRESOLUTION_960_540:
                        resolutionInfo.m_nWidth = 960
                        resolutionInfo.m_nHeight = 540
                    elif pResolution[k].value == SYDataDefine.SYResolutionEnum.SYRESOLUTION_1920_1080:
                        resolutionInfo.m_nWidth = 1920
                        resolutionInfo.m_nHeight = 1080
                    listLimitResolution.append(resolutionInfo)
                print("可用分辨率:" + str(len(listLimitResolution)))
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
            else:
                errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet, listLimitResolution

    # 开启数据流
    def start_device_stream(self, nDeviceId: int, streamType: SYDataDefine.SYStreamTypeEnum) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCode = dllLib.StartStreaming(c_uint(nDeviceId), c_int(streamType.value))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            return ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        return ErrCode.StatusCodeEnum.ERRORCODE_FAILED

    # 关闭数据流
    def stop_device_stream(self, nDeviceId: int) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCode = dllLib.StopStreaming(c_uint(nDeviceId))
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet

    # 切换数据流
    def change_device_stream(self, nDeviceId: int, streamType: DataDefine.StreamTypeEnum) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        errorCode = dllLib.ChangeStreaming(c_uint(nDeviceId), streamType)
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet

    # 获取最新一帧数据
    # nDeviceID 设备ID
    # pFrameData 最后一帧数据
    def get_last_frame_data(self, nDeviceId: int, pFrameData) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        errorCode = dllLib.GetLastFrameData(c_uint(nDeviceId), byref(pFrameData))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet

    # Obtain depth corresponding pseudo color(获取深度对应伪彩色)
    # nDeviceID 设备ID
    # nCount data count,(数据量(内存空间pDepth需要nCount * 2字节，pColor需要nCount * 3字节))
    # pDepth depth pointer(深度数据)
    # pColor Depth corresponds to pseudo color(24-bit RGB format),深度对应伪彩色(24位RGB格式)
    def get_depth_color(self, nDeviceId: int, nCount: int, pDepth: POINTER(c_ushort),
                        pColor: POINTER(c_ubyte)) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        errorCode = dllLib.GetDepthColor(c_uint(nDeviceId), c_int(nCount), pDepth, pColor)
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet

    # Obtain depth corresponding point cloud data(获取深度对应点云数据)
    # nDeviceID 设备ID
    # nWidth 宽度
    # nHeight 高度
    # pDepth Depth data pointer,深度数据
    # pPointCloud Depth corresponding point cloud data, externally allocated memory(深度对应点云数据, 由外部分配内存)
    # bUndistort 裁剪标志，true - 裁剪 false - 不裁剪
    def get_depth_point_cloud(self, nDeviceId: int, nWidth: int, nHeight: int, pDepth: POINTER(c_ushort), pPointCloud: POINTER(SYDataDefine.SYPointCloudData), bUndistort: bool) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        errorCode = dllLib.GetDepthPointCloud(c_uint(nDeviceId), c_int(nWidth), c_int(nHeight), pDepth, pPointCloud,
                                              c_bool(bUndistort))
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet

    # Get RGBD(获取RGBD)
    # nDeviceID 设备ID
    # nSourceDepthWidth Depth Source Data Width(源深度数据宽度)
    # nSourceDepthHeight Depth Source Data Height(源深度数据高度)
    # pSourceDepth depth source data pointer(源深度数据)
    # nSourceRGBWidth RGB source data width(源RGB数据宽度)
    # nSourceRGBHeight RGB source data height(源RGB数据高度)
    # pSourceRGB  RGB source data(源RGB数据)
    # nTargetWidth RGBD data width(RGBD数据宽度)
    # nTargetHeight  RGBD data height(RGBD数据高度)
    # pTargetDepth The depth data in RGBD is externally allocated memory,
    # and the data length is consistent with the source RGB length
    # (RGBD中的深度数据,由外部分配内存,数据长度与源RGB长度一致)
    # pTargetRGB RGB data in RGBD is externally allocated memory,
    # and the data length is consistent with the source RGB length
    # (RGBD中的RGB数据,由外部分配内存,数据长度与源RGB长度一致)
    def get_rgbd(self, nDeviceId: int, nSourceDepthWidth: int, nSourceDepthHeight: int, pSourceDepth: POINTER(c_ushort),
                 nSourceRGBWidth: int, nSourceRGBHeight: int, pSourceRGB: POINTER(c_ubyte), nTargetWidth: int,
                 nTargetHeight: int, pTargetDepth: POINTER(c_ushort),
                 pTargetRGB: POINTER(c_ubyte)) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        errorCode = dllLib.GetRGBD(c_uint(nDeviceId), c_int(nSourceDepthWidth), c_int(nSourceDepthHeight), pSourceDepth,
                                   c_int(nSourceRGBWidth), c_int(nSourceRGBHeight), pSourceRGB, c_int(nTargetWidth),
                                   c_int(nTargetHeight), pTargetDepth, pTargetRGB)
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet

    # 去畸变
    # nDeviceID 设备ID
    # pSource 待去畸变数据指针
    # nWidth 图像宽度
    # nHeight 图像高度
    # bDepth 是否是深度数据 / RGB数据
    # pTarget 去畸变结果数据指针，由外部分配内存, 数据长度与待去畸变数据指针长度一致
    def undistort(self, nDeviceId: int, pSource: POINTER(c_ushort), nWidth: int, nHeight: int, bDepth: bool,
                  pTarget: POINTER(c_ushort)) -> ErrCode.StatusCodeEnum:
        if nDeviceId == 0:
            return ErrCode.StatusCodeEnum.ERRORCODE_UNKOWNDEVICEID
        errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        errorCode = dllLib.Undistort(c_uint(nDeviceId), pSource, c_int(nWidth), c_int(nHeight), c_bool(bDepth), pTarget)
        if errorCode == SYDataDefine.SYErrorCodeEnum.SYERRORCODE_SUCCESS.code:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_SUCCESS
        else:
            errorCodeRet = ErrCode.StatusCodeEnum.ERRORCODE_FAILED
        return errorCodeRet
