import SynexensSDK
import SYDataDefine
import DataDefine
import ErrorCodeEnum
from ctypes import *
import cv2
import numpy as np
import threading
from SynexensSDK import print_t
from SynexensSDK import LogTypeEnum

pFrameData = POINTER(SYDataDefine.SYFrameData)()


def get_rgb_data(deviceControl):
    isLoop = True
    while isLoop:
        errorcode = deviceControl.get_last_frame_data(device_list[0].m_nDeviceId, pFrameData)
        if errorcode == ErrorCodeEnum.StatusCodeEnum.ERRORCODE_SUCCESS:
            objFrameData = pFrameData.contents
            # print_t(LogTypeEnum.DEBUG_T, "Count:{} BuffferLength:{}".format(objFrameData.m_nFrameCount, objFrameData.m_nBuffferLength))
            objFrameInfo = objFrameData.m_pFrameInfo.contents
            # print_t(LogTypeEnum.DEBUG_T,  "Height:{} Width:{}".format(objFrameInfo.m_nFrameHeight, objFrameInfo.m_nFrameWidth))

            data_pointer = cast(objFrameData.m_pData, POINTER(c_ubyte))
            # 将m_pData转换为numpy数组
            data_array = np.ctypeslib.as_array(data_pointer, shape=(objFrameData.m_nBuffferLength,))
            # print("data_array.size:" + str(data_array.size))
            img_yuyv_cvt = data_array.reshape(objFrameInfo.m_nFrameHeight, objFrameInfo.m_nFrameWidth, 2)
            # print("img_yuyv_cvt.shape:" + str(img_yuyv_cvt.shape))

            yuyv_array = np.frombuffer(img_yuyv_cvt, dtype=np.uint8)
            yuyv_image = yuyv_array.reshape((objFrameInfo.m_nFrameHeight, objFrameInfo.m_nFrameWidth, 2))
            rgb_image = cv2.cvtColor(yuyv_image, cv2.COLOR_YUV2BGR_YUYV)

            # cv2.imwrite('C:\\Users\\makerjia\\Desktop\\image' + str(index) + '.jpg', rgb_image)
            cv2.imshow('RGB', rgb_image)
            key = cv2.waitKey(100)

            # If the ESC key is pressed, the window will be closed
            if key == 27:
                isLoop = False
                cv2.destroyAllWindows()


def get_depth_ir_data(deviceControl):
    isLoop = True
    while isLoop:
        # Obtain data continuously through polling, pFrameData is a SYFrameData type Pointer
        errorcode = deviceControl.get_last_frame_data(
            device_list[0].m_nDeviceId, pFrameData)
        if errorcode == ErrorCodeEnum.StatusCodeEnum.ERRORCODE_SUCCESS:
            # Get SYFrameData Instance Object
            objFrameData = pFrameData.contents
            # print_t(LogTypeEnum.DEBUG_T, "count:{}  BuffferLength:{}".format(objFrameData.m_nFrameCount, objFrameData.m_nBuffferLength))
            # Get SYFrameInfo Instance Object
            objFrameInfo = objFrameData.m_pFrameInfo.contents
            # print_t(LogTypeEnum.DEBUG_T, "Height:{}  Width:{}".format(objFrameInfo.m_nFrameHeight,objFrameInfo.m_nFrameWidth))

            # If start depthIr stream,the value of nFrameCount should be 2,Obtain the data of Depth and IR through pointer offset
            nPos = 0
            key = 0
            for i in range(objFrameData.m_nFrameCount):
                if objFrameData.m_pFrameInfo[i].m_frameType == SYDataDefine.SYFrameTypeEnum.SYFRAMETYPE_DEPTH.value:
                    nFrameHeight = objFrameData.m_pFrameInfo[i].m_nFrameHeight
                    nFrameWidth = objFrameData.m_pFrameInfo[i].m_nFrameWidth
                    nCount = nFrameHeight * nFrameWidth
                    pDepth = (c_ushort * nCount)()  # Depth data pointer
                    offset = nPos
                    ptr_void_new = c_void_p(objFrameData.m_pData + offset)  # It is necessary to set the offset under c_void_p
                    ptr_new = cast(ptr_void_new, POINTER(c_ushort))
                    memmove(pDepth, ptr_new, sizeof(c_ushort) * nCount)

                    pColor = (c_ubyte * nCount * 3)()
                    # similar to cv::Mat depthimg = cv::Mat(nFrameHeight, nFrameWidth, CV_8UC3);
                    depthimg = np.zeros((nFrameHeight, nFrameWidth, 3), dtype=np.uint8)
                    # Obtain the pseudo color corresponding to the depth
                    getDepthColorErrCode = deviceControl.get_depth_color(device_list[0].m_nDeviceId, nCount, pDepth, pColor)
                    if getDepthColorErrCode == ErrorCodeEnum.StatusCodeEnum.ERRORCODE_SUCCESS:
                        memmove(depthimg.ctypes.data, pColor, objFrameData.m_pFrameInfo[i].m_nFrameHeight * objFrameData.m_pFrameInfo[i].m_nFrameWidth * 3)
                        cv2.imshow("depth_{}".format(device_list[0].m_nDeviceId), depthimg)
                        key = cv2.waitKey(100)

                    nPos += objFrameData.m_pFrameInfo[i].m_nFrameHeight * objFrameData.m_pFrameInfo[i].m_nFrameWidth * sizeof(c_short)
                elif objFrameData.m_pFrameInfo[
                    i].m_frameType == SYDataDefine.SYFrameTypeEnum.SYFRAMETYPE_IR.value:
                    nFrameHeight = objFrameData.m_pFrameInfo[i].m_nFrameHeight
                    nFrameWidth = objFrameData.m_pFrameInfo[i].m_nFrameWidth
                    nCount = nFrameHeight * nFrameWidth
                    pIR = (c_ushort * nCount)()
                    offset = nPos
                    ptr_void_new = c_void_p(objFrameData.m_pData + offset)
                    ptr_new = cast(ptr_void_new, POINTER(c_ushort))
                    memmove(pIR, ptr_new, sizeof(c_ushort) * nCount)

                    gray16 = np.frombuffer(pIR, dtype=np.uint16).reshape(nFrameHeight, nFrameWidth)
                    # cv2.normalize is a function in the OpenCV library used to normalize the values of a given image. Normalization is the process of adjusting the pixel values of an image to a new range to improve its contrast and brightness.
                    gray8 = cv2.normalize(gray16,None,0,255, cv2.NORM_MINMAX, cv2.CV_8U)
                    cv2.imshow("ir_{}".format(device_list[0].m_nDeviceId), gray8)
                    key = cv2.waitKey(100)

                    nPos += nFrameHeight * nFrameWidth * sizeof(c_short)

                # Close the window if the ESC key is pressed
                if key == 27:
                    isLoop = False
                    cv2.destroyAllWindows()


if __name__ == '__main__':
    deviceControl = SynexensSDK.DeviceControl()
    device_list: list[DataDefine.DeviceInfo] = []
    # 1.step find device
    errcode = deviceControl.find_device(device_list)
    print_t(LogTypeEnum.DEBUG_T, "FindDevice:{}".format(errcode.errmsg))
    if errcode == ErrorCodeEnum.StatusCodeEnum.ERRORCODE_SUCCESS:
        for i in range(len(device_list)):
            # 2.step open device
            errCode = deviceControl.open_device(device_list[i].m_nDeviceId)
            print_t(LogTypeEnum.DEBUG_T, "OpenDevice:{}".format(errCode.errmsg))

            # 3.step Get Available Resolution
            listLimitResolution: list[DataDefine.ResolutionInfo] = list()
            result = deviceControl.get_device_usable_resolution(device_list[i].m_nDeviceId, DataDefine.GraphTypeEnum.GRAPHTYPE_DEPTH, listLimitResolution)
            print_t(LogTypeEnum.DEBUG_T, "Get Available Resolution:{}".format(result[0].errmsg))

            if result[0] == ErrorCodeEnum.StatusCodeEnum.ERRORCODE_SUCCESS:
                listLimitResolution = result[1]
                print_t(LogTypeEnum.DEBUG_T, "Available Resolution Count:{}".format(len(listLimitResolution)))

                if len(listLimitResolution) > 0:
                    defaultResolution = listLimitResolution[1]
                    print_t(LogTypeEnum.DEBUG_T, "width:{} height:{}".format(defaultResolution.m_nWidth, defaultResolution.m_nHeight))

                    # 4.step set resolution
                    errcode = deviceControl.set_resolution(device_list[0].m_nDeviceId, DataDefine.GraphTypeEnum.GRAPHTYPE_DEPTH, defaultResolution)
                    errcode = deviceControl.set_resolution(device_list[0].m_nDeviceId, DataDefine.GraphTypeEnum.GRAPHTYPE_IR, defaultResolution)
                    print_t(LogTypeEnum.DEBUG_T, "Set resolution:{}".format(errcode.errmsg))

                    # 5.step start stream
                    errcode = deviceControl.start_device_stream(device_list[0].m_nDeviceId, SYDataDefine.SYStreamTypeEnum.SYSTREAMTYPE_DEPTHIR)
                    print_t(LogTypeEnum.DEBUG_T, "start stream:{}".format(errcode.errmsg))

                    t1 = threading.Thread(target=get_depth_ir_data, args=(deviceControl,), name='getDataThread')
                    t1.start()
                    t1.join()

        for i in range(len(device_list)):
            deviceControl.close_device(device_list[i].m_nDeviceId)

        deviceControl.uninit_sdk()
